"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

setup(
    name="CITransitionSelectorSample",
    app=["main.py"],
    data_files=[
        "English.lproj",
        "Blank.jpg",
        "Frog.jpg",
        "Mask.jpg",
        "Rose.jpg",
        "Shading.tiff",
    ],
    setup_requires=["py2app", "pyobjc-framework-Cocoa", "pyobjc-framework-Quartz"],
)



"""
Python mapping for the PDFKit framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import AppKit
import objc
from Quartz.PDFKit import _metadata, _PDFKit

# XXX: Moved to topelevel framework in 10.15

sys.modules["Quartz.PDFKit"] = mod = objc.ObjCLazyModule(
    "Quartz.PDFKit",
    "com.apple.PDFKit",
    objc.pathForFramework(
        "/System/Library/Frameworks/Quartz.framework/Frameworks/PDFKit.framework"
    ),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (_PDFKit, AppKit),
)


del sys.modules["Quartz.PDFKit._metadata"]



import Cocoa
import objc
import Quartz
from objc import super


class MyPDFDocument(Cocoa.NSDocument):
    _outline = objc.ivar()
    _searchResults = objc.ivar()

    _drawer = objc.IBOutlet()
    _noOutlineText = objc.IBOutlet()
    _outlineView = objc.IBOutlet()
    _pdfView = objc.IBOutlet()
    _searchProgress = objc.IBOutlet()
    _searchTable = objc.IBOutlet()

    def dealloc(self):
        Cocoa.NSNotificationCenter.defaultCenter().removeObserver_(self)
        self._searchResults = None
        super(MyPDFDocument, self).dealloc()

    def windowNibName(self):
        return "MyDocument"

    def windowControllerDidLoadNib_(self, controller):
        super(MyPDFDocument, self).windowControllerDidLoadNib_(controller)

        if self.fileName():
            pdfDoc = Quartz.PDFDocument.alloc().initWithURL_(
                Cocoa.NSURL.fileURLWithPath_(self.fileName())
            )
            self._pdfView.setDocument_(pdfDoc)

        # Page changed notification.
        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, "pageChanged:", Quartz.PDFViewPageChangedNotification, self._pdfView
        )

        # Find notifications.
        center = Cocoa.NSNotificationCenter.defaultCenter()
        center.addObserver_selector_name_object_(
            self,
            "startFind:",
            Quartz.PDFDocumentDidBeginFindNotification,
            self._pdfView.document(),
        )
        center.addObserver_selector_name_object_(
            self,
            "findProgress:",
            Quartz.PDFDocumentDidEndPageFindNotification,
            self._pdfView.document(),
        )
        center.addObserver_selector_name_object_(
            self,
            "endFind:",
            Quartz.PDFDocumentDidEndFindNotification,
            self._pdfView.document(),
        )

        # Set self to be delegate (find).
        self._pdfView.document().setDelegate_(self)

        # Get outline.
        self._outline = self._pdfView.document().outlineRoot()
        if self._outline is not None:
            # Remove text that says, "No outline."
            self._noOutlineText.removeFromSuperview()
            self._noOutlineText = None

            # Force it to load up.
            self._outlineView.reloadData()

        else:
            # Remove outline view (leaving instead text that says,
            # "No outline.").
            self._outlineView.enclosingScrollView().removeFromSuperview()
            self._outlineView = None

        # Open drawer.
        self._drawer.open()

        # Size the window.
        windowSize = self._pdfView.rowSizeForPage_(self._pdfView.currentPage())

        if (self._pdfView.displayMode() & 0x01) and (
            self._pdfView.document().pageCount() > 1
        ):
            windowSize.width += Cocoa.NSScroller.scrollerWidth()
        controller.window().setContentSize_(windowSize)

    def dataRepresentationOfType_(self, aType):
        return None

    def loadDataRepresentation_ofType_(self, data, aType):
        return True

    @objc.IBAction
    def toggleDrawer_(self, sender):
        self._drawer.toggle_(self)

    @objc.IBAction
    def takeDestinationFromOutline_(self, sender):
        # Get the destination associated with the search result list.
        # Tell the PDFView to go there.
        self._pdfView.goToDestination_(
            sender.itemAtRow_(sender.selectedRow()).destination()
        )

    @objc.IBAction
    def displaySinglePage_(self, sender):
        # Display single page mode.
        if self._pdfView.displayMode() > Quartz.kPDFDisplaySinglePageContinuous:
            self._pdfView.setDisplayMode_(self._pdfView.displayMode() - 2)

    @objc.IBAction
    def displayTwoUp_(self, sender):
        #  Display two-up.
        if self._pdfView.displayMode() < Quartz.kPDFDisplayTwoUp:
            self._pdfView.setDisplayMode_(self._pdfView.displayMode() + 2)

    def pageChanged_(self, notification):
        # Skip out if there is no outline.
        if self.pdfView.document().outlineRoot() is None:
            return

        # What is the new page number (zero-based).
        newPageIndex = self._pdfView.document().indexForPage_(
            self._pdfView.currentPage()
        )

        # Walk outline view looking for best firstpage number match.
        newlySelectedRow = -1
        numRows = self._outlineView.numberOfRows()
        for i in range(numRows):
            outlineItem = self._outlineView.itemAtRow_(i)

            if (
                self._pdfView.document().indexForPage_(outlineItem.destination().page())
                == newPageIndex
            ):

                newlySelectedRow = i
                self._outlineView.selectRow_byExtendingSelection_(
                    newlySelectedRow, False
                )
                break

            elif (
                self._pdfView.document().indexForPage_(
                    outlineItem.destionation().page()
                )
                > newPageIndex
            ):
                newlySelectedRow = i - 1
                self._outlineView.selectRow_byExtendingSelection_(
                    newlySelectedRow, False
                )
                break

        # Auto-scroll.
        if newlySelectedRow != -1:
            self._outlineView.scrollRowToVisible_(newlySelectedRow)

    def doFind_(self, sender):
        if self._pdfView.document().isFinding():
            self._pdfView.document().cancelFindString()

        # Lazily allocate _searchResults.
        if self._searchResults is None:
            self._searchResults = Cocoa.NSMutableArray.arrayWithCapacity_(10)

        self._pdfView.document().beginFindString_withOptions_(
            sender.stringValue(), Cocoa.NSCaseInsensitiveSearch
        )

    def startFind_(self, notification):
        # Empty arrays.
        self._searchResults.removeAllObjects()
        self._searchTable.reloadData()
        self._searchProgress.startAnimation_(self)

    def findProgress_(self, notification):
        pageIndex = notification.userInfo().objectForKey_(
            "PDFDocumentPageIndex"
        )  # .doubleValue()
        self._searchProgress.setDoubleValue_(
            pageIndex / self._pdfView.document().pageCount()
        )

    def didMatchString_(self, instance):
        # Add page label to our array.
        self._searchResults.addObject_(instance.copy())
        self._searchTable.reloadData()

    def endFind_(self, notification):
        self._searchProgress.stopAnimation_(self)
        self._searchProgress.setDoubleValue_(0)

    #  The table view is used to hold search results.  Column 1 lists the
    # page number for the search result,  column two the section in the PDF
    # (x-ref with the PDF outline) where the result appears.

    def numberOfRowsInTableView_(self, aTableView):
        if self._searchResults is None:
            return 0
        return self._searchResults.count()

    def tableView_objectValueForTableColumn_row_(self, aTableView, theColumn, rowIndex):

        if theColumn.identifier() == "page":
            return (
                self._searchResults.objectAtIndex_(rowIndex)
                .pages()
                .objectAtIndex_(0)
                .label()
            )

        elif theColumn.identifier() == "section":
            value = self._pdfView.document().outlineItemForSelection_(
                self._searchResults.objectAtIndex_(rowIndex)
            )

            if value is None:
                return None

            return value.label()

        else:
            return None

    def tableViewSelectionDidChange_(self, notification):
        # What was selected.  Skip out if the row has not changed.
        rowIndex = notification.object().selectedRow()
        if rowIndex >= 0:
            self._pdfView.setCurrentSelection_(
                self._searchResults.objectAtIndex_(rowIndex)
            )
            self._pdfView.centerSelectionInVisibleArea_(self)

    # The outline view is for the PDF outline.  Not all PDF's have an outline.
    def outlineView_numberOfChildrenOfItem_(self, outlineView, item):
        if item is None:
            if self._outline is not None:
                return self._outline.numberOfChildren()
            else:
                return 0

        else:
            return item.numberOfChildren()

    def outlineView_child_ofItem_(self, outlineView, index, item):
        if item is None:
            if self._outline is not None:
                return self._outline.childAtIndex_(index).retain()
            else:
                return None

        else:
            return item.childAtIndex_(index).retain()

    def outlineView_isItemExpandable_(self, outlineView, item):
        if item is None:
            if self._outline:
                return self._outline.numberOfChildren() > 0

            else:
                return False

        else:
            return item.numberOfChildren() > 0

    def outlineView_objectValueForTableColumn_byItem_(
        self, outlineView, tableColumn, item
    ):
        return item.label()



"""
Wrappers for the "Quartz" related frameworks on macOS. These frameworks
provide a number of graphics related API's.

The frameworks wrapped by this package are:

   * CoreGraphics - 2D Graphics, based on the PDF model

   * ImageIO - Reading and writing images

   * QuartzComposer - Working with "Quartz Composer" compositions

   * QuartzCore  - Image processing and video image manipulation

   * QuarzFilters - Image effects

   * ImageKit - iPhoto-like views

   * PDFKit - Working with PDF files

   * CoreVideo - Managing digital video

All frameworks can be accessed by importing the 'Quartz' module.

These wrappers don't include documentation, please check Apple's documention
for information on how to use this framework and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks

NOTE: The actual wrappers are subpackages of ``Quartz``, they are not toplevel
packages to avoid name clashes with Apple provided wrappers for CoreGraphics.

WARNING: Running the unittests will change your display settings during the
testrun, which will probably mess up your window layout.

NEWS
====

2.4
---

* Add wrapper for ``CGBitmapContextCreateWithData``

"""

import os

from pyobjc_setup import Extension, setup

subpackages = [
    "Quartz.%s" % (fn,)
    for fn in os.listdir("Lib/Quartz")
    if os.path.exists(os.path.join("Lib/Quartz", fn, "__init__.py"))
]

VERSION = '6.2'

setup(
    name="pyobjc-framework-Quartz",
    description="Wrappers for the Quartz frameworks on macOS",
    packages=["Quartz"] + subpackages,
    ext_modules=[
        # CoreVideo
        Extension(
            "Quartz.CoreVideo._CVPixelBuffer",
            ["Modules/_CVPixelBuffer.m"],
            py_limited_api=True,
        ),
        # CoreGraphics
        Extension(
            "Quartz.CoreGraphics._inlines",
            ["Modules/_CoreGraphics_inlines.m"],
            py_limited_api=True,
        ),
        Extension(
            "Quartz.CoreGraphics._callbacks",
            ["Modules/_callbacks.m"],
            extra_compile_args=["-Wno-deprecated-declarations"],
            # py_limited_api=True,
        ),
        Extension(
            "Quartz.CoreGraphics._doubleindirect",
            ["Modules/_doubleindirect.m"],
            py_limited_api=True,
        ),
        Extension(
            "Quartz.CoreGraphics._sortandmap",
            ["Modules/_sortandmap.m"],
            py_limited_api=True,
        ),
        Extension(
            "Quartz.CoreGraphics._coregraphics",
            ["Modules/_coregraphics.m"],
            extra_link_args=["-framework", "ApplicationServices"],
            # py_limited_api=True,
        ),
        Extension(
            "Quartz.ImageKit._imagekit",
            ["Modules/_imagekit.m"],
            extra_link_args=["-framework", "Quartz"],
            py_limited_api=True,
        ),
        Extension(
            "Quartz.PDFKit._PDFKit",
            ["Modules/_PDFKit.m"],
            extra_link_args=["-framework", "Quartz"],
            py_limited_api=True,
        ),
        Extension(
            "Quartz.QuartzCore._quartzcore",
            ["Modules/_quartzcore.m"],
            extra_link_args=["-framework", "QuartzCore"],
            py_limited_api=True,
        ),
        Extension(
            "Quartz.QuickLookUI._QuickLookUI",
            ["Modules/_QuickLookUI.m"],
            extra_link_args=["-framework", "Quartz"],
            py_limited_api=True,
        ),
    ],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
    long_description=__doc__,
)



# This file is generated by objective.metadata
#
# Last update: Wed Aug  7 07:43:21 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$$"""
enums = """$QLPreviewViewStyleCompact@1$QLPreviewViewStyleNormal@0$"""
misc.update({})
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"NSObject",
        b"acceptsPreviewPanelControl:",
        {"retval": {"type": b"Z"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"beginPreviewPanelControl:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"endPreviewPanelControl:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"numberOfPreviewItemsInPreviewPanel:",
        {
            "required": True,
            "retval": {"type": sel32or64(b"i", b"q")},
            "arguments": {2: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"preparePreviewOfFileAtURL:completionHandler:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    },
                    "type": "@?",
                },
            },
        },
    )
    r(
        b"NSObject",
        b"preparePreviewOfSearchableItemWithIdentifier:queryString:completionHandler:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    },
                    "type": "@?",
                },
            },
        },
    )
    r(
        b"NSObject",
        b"previewItemDisplayState",
        {"required": False, "retval": {"type": b"@"}},
    )
    r(b"NSObject", b"previewItemTitle", {"required": False, "retval": {"type": b"@"}})
    r(b"NSObject", b"previewItemURL", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"previewPanel:handleEvent:",
        {
            "required": False,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"previewPanel:previewItemAtIndex:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"i", b"q")}},
        },
    )
    r(
        b"NSObject",
        b"previewPanel:sourceFrameOnScreenForPreviewItem:",
        {
            "required": False,
            "retval": {
                "type": sel32or64(
                    b"{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                    b"{CGRect={CGPoint=dd}{CGSize=dd}}",
                )
            },
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"previewPanel:transitionImageForPreviewItem:contentRect:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {
                    "type": sel32or64(
                        b"^{_NSRect={_NSPoint=ff}{_NSSize=ff}}",
                        b"^{CGRect={CGPoint=dd}{CGSize=dd}}",
                    )
                },
            },
        },
    )
    r(
        b"NSObject",
        b"setPreviewItemDisplayState:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setPreviewItemTitle:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setPreviewItemURL:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"QLPreviewPanel",
        b"enterFullScreenMode:withOptions:",
        {"retval": {"type": b"Z"}},
    )
    r(b"QLPreviewPanel", b"isInFullScreenMode", {"retval": {"type": b"Z"}})
    r(b"QLPreviewPanel", b"sharedPreviewPanelExists", {"retval": {"type": b"Z"}})
    r(b"QLPreviewView", b"autostarts", {"retval": {"type": b"Z"}})
    r(b"QLPreviewView", b"setAutostarts:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"QLPreviewView",
        b"setShouldCloseWithWindow:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"QLPreviewView", b"shouldCloseWithWindow", {"retval": {"type": b"Z"}})
finally:
    objc._updatingMetadata(False)
protocols = {
    "QLPreviewPanelController": objc.informal_protocol(
        "QLPreviewPanelController",
        [
            objc.selector(
                None, b"beginPreviewPanelControl:", b"v@:@", isRequired=False
            ),
            objc.selector(
                None, b"acceptsPreviewPanelControl:", b"Z@:@", isRequired=False
            ),
            objc.selector(None, b"endPreviewPanelControl:", b"v@:@", isRequired=False),
        ],
    )
}
expressions = {}

# END OF FILE



import Cocoa
import objc
import TLayerDemo


class AppDelegate(Cocoa.NSObject):
    shadowDemo = objc.ivar()

    def applicationDidFinishLaunching_(self, notification):
        self.showTLayerDemoWindow_(self)

    @objc.IBAction
    def showTLayerDemoWindow_(self, sender):
        if self.shadowDemo is None:
            self.shadowDemo = TLayerDemo.TLayerDemo.alloc().init()

        self.shadowDemo.window().orderFront_(self)

    def applicationShouldTerminateAfterLastWindowClosed_(self, app):
        return True



import BitmapContext
import Cocoa
import objc
import Quartz
import Utilities

# We're using a function that isn't made available through a wrapper, just
# load it manually:
if not hasattr(Quartz, "PMCGImageCreateWithEPSDataProvider"):
    functions = [("PMCGImageCreateWithEPSDataProvider", b"@@@")]
    import AppKit

    d = {}
    objc.loadBundleFunctions(AppKit.__bundle__, d, functions)

    if "PMCGImageCreateWithEPSDataProvider" in d:
        PMCGImageCreateWithEPSDataProvider = d["PMCGImageCreateWithEPSDataProvider"]
    else:
        print("PMCGImageCreateWithEPSDataProvider doesn't exist")


def getEPSBBox(epspath):
    try:
        fp = open(epspath, "rU")
    except IOError:
        return Quartz.CGRectZero

    try:
        #  This is a VERY poor man's EPS DSC parser, here just so that
        #  this sample code can handle simple EPS files. It is
        #  simple but very inefficient. In addition it does not ensure
        #  that the DSC comments are at the beginning of a line,
        #  nor does it handle (atend) style comments at all.
        #  It will simply find the first occurance of a
        #  %%BoundingBox comment and if it is of the typical
        # form, it will obtain the bounding box data.
        #
        for ln in fp:
            if ln.startswith("%%BoundingBox:"):
                fields = ln.split()[1:]
                if len(fields) >= 4:
                    llx = int(fields[0])
                    lly = int(fields[1])
                    urx = int(fields[2])
                    ury = int(fields[3])
                    return Quartz.CGRectMake(llx, lly, urx - llx, ury - lly)
    finally:
        fp.close()

    return Quartz.CGRectZero


def createEPSPreviewImage(url):
    # The CGImage used as the preview needs to have the
    # same width and height as the EPS data it will
    # be associated with. This sample code doesn't attempt
    # to use any preview image associated with the EPS
    # data but instead simply draws a box of an appropriate
    # size. Your code would most likely create an image
    # that reflects a PICT or TIFF preview present in the
    # EPS data.
    result, path = Cocoa.CFURLGetFileSystemRepresentation(url, True, None, 1024)
    if not result:
        print("Couldn't get the path for EPS file!")
        return None

    path = path.rstrip(b"\0")

    epsRect = getEPSBBox(path)
    # Check whether the EPS bounding box is empty.
    if epsRect == Quartz.CGRectZero:
        print("Couldn't find BoundingBox comment!")
        return None

    wantDisplayColorSpace = False
    needsTransparentBitmap = True
    # Create a bitmap context to draw to in order to
    # create the preview image. Use the routine
    # createRGBBitmapContext from the earlier chapter.
    bitmapContext = BitmapContext.createRGBBitmapContext(
        epsRect.size.width,
        epsRect.size.height,
        wantDisplayColorSpace,
        needsTransparentBitmap,
    )
    if bitmapContext is None:
        print("Couldn't create bitmap context")
        return None

    epsRect.origin.x = epsRect.origin.y = 0
    # Draw the contents of the preview. The preview consists
    # of two lines and a stroke around the bounding box. One
    # of the two lines is drawn from the lower-left corner to
    # the upper-right corner of the bounding box and the other
    # line is from the lower-right corner to the upper-left
    # corner of the bounding box.
    Quartz.CGContextBeginPath(bitmapContext)
    Quartz.CGContextMoveToPoint(bitmapContext, 0, 0)
    Quartz.CGContextAddLineToPoint(
        bitmapContext, epsRect.size.width, epsRect.size.height
    )
    Quartz.CGContextMoveToPoint(bitmapContext, epsRect.size.width, 0)
    Quartz.CGContextAddLineToPoint(bitmapContext, 0, epsRect.size.height)
    Quartz.CGContextStrokePath(bitmapContext)
    # Stroke the bounding rectangle, inset so that the stroke is
    # completely contained in the EPS bounding rect.
    Quartz.CGContextStrokeRect(bitmapContext, Quartz.CGRectInset(epsRect, 0.5, 0.5))

    # Now create an image from the bitmap raster data. This image
    # has a data provider that releases the image raster data when
    # the image is released. Use the createImageFromBitmapContext
    # from Chapter 12. Calling createImageFromBitmapContext
    # gives up ownership of the raster data used by the context.
    epsPreviewImage = BitmapContext.createImageFromBitmapContext(bitmapContext)

    if epsPreviewImage is None:
        print("Couldn't create preview image!")
        return None

    return epsPreviewImage


# This technique of handling EPS data is available in
# macOS v10.1 and later and is one alternative method
# of supporting EPS data during printing as compared to
# converting EPS data to PDF data using CGPSConverter which
# is only available in Panther and later.
def createCGEPSImage(url):
    previewImage = createEPSPreviewImage(url)
    if previewImage is None:
        print("Couldn't create EPS preview!")
        return None

    # It is important that the data provider supplying the
    # EPS data conform to the Quartz guidelines for data providers
    # and is able to provide the data until the data releaser function
    # is called. If you have a custom data provider, you need
    # to follow these guidelines since your data provider
    # is not necessarily called before you release the image
    # that uses the provider.
    epsDataProvider = Quartz.CGDataProviderCreateWithURL(url)
    if epsDataProvider is None:
        print("Couldn't create EPS data provider!")
        return None

    # Create the hybrid CGImage that contains the preview image
    # and the EPS data. Note that the data provider isn't
    # called during image creation but at some later point in time.

    epsImage = PMCGImageCreateWithEPSDataProvider(epsDataProvider, previewImage)
    # The preview image and data provider are no longer needed
    # because Quartz retains them and this code doesn't
    # require them further.
    del previewImage
    del epsDataProvider

    if epsImage is None:
        print("Couldn't create EPS hybrid image!")
        return None

    return epsImage


def drawEPSDataImage(context, url):
    # Create the a CGImage that has EPS data associated with it.
    epsDataImage = createCGEPSImage(url)
    if epsDataImage is None:
        return

    # Create a destination rectangle at the location
    # to draw the EPS document. The size of the rect is scaled
    # down to 1/2 the size of the EPS graphic.
    destinationRect = Quartz.CGRectMake(
        100,
        100,
        Quartz.CGImageGetWidth(epsDataImage),
        Quartz.CGImageGetHeight(epsDataImage),
    )
    # Draw the image to the destination. When the EPS
    # data associated with the image is sent to a PostScript
    # printer, the EPS bounding box is mapped to this
    # destination rectangle, translated and scaled as necessary.
    Quartz.CGContextDrawImage(context, destinationRect, epsDataImage)

    # Draw the image a second time. This time the image is
    # rotated by 45 degrees and scaled by an additional scaling factor
    # of 0.5 in the x dimension. The center point of this image coincides
    # with the center point of the earlier drawing.
    Quartz.CGContextTranslateCTM(
        context,
        destinationRect.origin.x + destinationRect.size.width / 2,
        destinationRect.origin.y + destinationRect.size.height / 2,
    )
    Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(45))
    Quartz.CGContextScaleCTM(context, 0.5, 1)
    Quartz.CGContextTranslateCTM(
        context,
        -(destinationRect.origin.x + destinationRect.size.width / 2),
        -(destinationRect.origin.y + destinationRect.size.height / 2),
    )
    Quartz.CGContextDrawImage(context, destinationRect, epsDataImage)



import Cocoa
import MyHazeFilter
import objc
import Quartz


class HazeFilterView(Cocoa.NSView):
    filter = objc.ivar()  # noqa: A003
    distance = objc.ivar(type=objc._C_FLT)
    slope = objc.ivar(type=objc._C_FLT)

    def distanceSliderChanged_(self, sender):
        self.distance = sender.floatValue()
        self.setNeedsDisplay_(True)

    def slopeSliderChanged_(self, sender):
        self.slope = sender.floatValue()
        self.setNeedsDisplay_(True)

    def drawRect_(self, rect):
        cg = Quartz.CGRectMake(
            Cocoa.NSMinX(rect),
            Cocoa.NSMinY(rect),
            Cocoa.NSWidth(rect),
            Cocoa.NSHeight(rect),
        )

        context = Cocoa.NSGraphicsContext.currentContext().CIContext()

        if self.filter is None:
            # make sure initialize is called
            MyHazeFilter.MyHazeFilter.pyobjc_classMethods.class__()

            url = Cocoa.NSURL.fileURLWithPath_(
                Cocoa.NSBundle.mainBundle().pathForResource_ofType_("CraterLake", "jpg")
            )
            self.filter = Quartz.CIFilter.filterWithName_("MyHazeRemover")
            self.filter.setValue_forKey_(
                Quartz.CIImage.imageWithContentsOfURL_(url), "inputImage"
            )

            self.filter.setValue_forKey_(
                Quartz.CIColor.colorWithRed_green_blue_(0.7, 0.9, 1), "inputColor"
            )

        self.filter.setValue_forKey_(self.distance, "inputDistance")
        self.filter.setValue_forKey_(self.slope, "inputSlope")

        if context is not None:
            context.drawImage_atPoint_fromRect_(
                self.filter.valueForKey_("outputImage"), cg.origin, cg
            )



import BitmapContext
import Cocoa
import ColorAndGState
import CoordinateSystem
import DrawingBasics
import EPSPrinting
import ImageMasking
import Images
import PathDrawing
import PatternDrawing
import QuartzTextDrawing
import Shadings
import ShadowsAndTransparencyLayers
import UIHandling
import Utilities

# Defines
kCatPDF = "Kitty.pdf"
kPDFForBlendMode = "blendmode.pdf"

kOurJPEG = "Poot.jpg"
kQTImage = "ptlobos.tif"
kOurSubstituteJPG = "LyingOnDeckNoProfile.JPG"
kOurEPS = "imageturkey.eps"

RAW_IMAGE_WIDTH = 400
RAW_IMAGE_HEIGHT = 300
kRawColorImage = "image-400x300x24.raw"
kOtherColorImage = "otherimage-400x300x24.raw"

MASKING_IMAGE_WIDTH = 400
MASKING_IMAGE_HEIGHT = 259
kMaskingImage = "400x259x8.bw.raw"

noOffScreen = 0
bitmapOffScreen = 1
layerOffScreen = 2

# Cache info for GetURL
_mainBundle = None
_urlMap = {}


def GetURL(name):
    """
    Returns the CFURLRef for an embeded resource, or None of that cannot be found.
    """

    global _mainBundle
    if _mainBundle is None:
        _mainBundle = Utilities.getAppBundle()

    mainBundle = _mainBundle
    if mainBundle is not None:
        if name in _urlMap:
            return _urlMap[name]

        url = Cocoa.CFBundleCopyResourceURL(mainBundle, name, None, None)
        _urlMap[name] = url
    else:
        print("Can't get the app bundle!")
        return

    if url is None:
        print("Couldn't get URL for %r" % (name,))

    return url


#
# Helper functions for drawing
#


def callPDFDrawProc(context, proc, pdfFile):
    ourPDFurl = GetURL(pdfFile)

    if ourPDFurl:
        proc(context, ourPDFurl)


def doDrawJPEGFile(context):
    ourJPEGurl = GetURL(kOurJPEG)

    if ourJPEGurl is not None:
        Images.drawJPEGImage(context, ourJPEGurl)


def doRawImageFileWithURL(context):
    url = GetURL(kRawColorImage)

    if url is not None:
        Images.drawImageFromURL(
            context, url, RAW_IMAGE_WIDTH, RAW_IMAGE_HEIGHT, 8, True
        )
        # 8 bits per component, isColor = True


def doRawImageFileWithCallbacks(context):
    url = GetURL(kRawColorImage)

    if url is not None:
        Images.doImageWithCallbacksCreatedFromURL(
            context, url, RAW_IMAGE_WIDTH, RAW_IMAGE_HEIGHT, 8, True
        )
        # 8 bits per component, isColor = True


def doDrawImageWithCGImageSource(context):
    url = GetURL(kOurJPEG)
    if url is not None:
        Images.drawImageWithCGImageDataSource(context, url)


def doIncrementalImage(context):
    url = GetURL(kOurJPEG)

    if url is not None:
        Images.doIncrementalImageWithURL(context, url)


def doQTImage(context):
    url = GetURL(kQTImage)

    if url is not None:
        Images.drawQTImageWithQuartz(context, url)


def doJPEGDocumentWithMultipleProfiles(context):
    url = GetURL(kOurSubstituteJPG)

    if url is not None:
        Images.drawJPEGDocumentWithMultipleProfiles(context, url)


def doMaskImageWithMask(context):
    theImageToMaskURL = GetURL(kOtherColorImage)
    theMaskingImageURL = GetURL(kMaskingImage)

    if theImageToMaskURL is not None and theMaskingImageURL is not None:
        ImageMasking.doMaskImageWithMaskFromURL(
            context,
            theImageToMaskURL,
            RAW_IMAGE_WIDTH,
            RAW_IMAGE_HEIGHT,
            8,
            theMaskingImageURL,
            MASKING_IMAGE_WIDTH,
            MASKING_IMAGE_HEIGHT,
        )


def doMaskImageWithGrayImage(context):
    theImageToMaskURL = GetURL(kOtherColorImage)
    theMaskingImageURL = GetURL(kMaskingImage)

    if theImageToMaskURL is not None and theMaskingImageURL is not None:
        ImageMasking.doMaskImageWithGrayImageFromURL(
            context,
            theImageToMaskURL,
            RAW_IMAGE_WIDTH,
            RAW_IMAGE_HEIGHT,
            8,
            theMaskingImageURL,
            MASKING_IMAGE_WIDTH,
            MASKING_IMAGE_HEIGHT,
        )


def doImageMaskedWithColor(context):
    url = GetURL(kOtherColorImage)

    if url is not None:
        ImageMasking.doMaskImageWithColorFromURL(
            context, url, RAW_IMAGE_WIDTH, RAW_IMAGE_HEIGHT, True
        )


def exportImageMaskedWithImage(context):
    theImageToMaskURL = GetURL(kOtherColorImage)
    theMaskingImageURL = GetURL(kMaskingImage)

    if theImageToMaskURL is not None and theMaskingImageURL is not None:
        ImageMasking.exportImageWithMaskFromURLWithDestination(
            context,
            theImageToMaskURL,
            RAW_IMAGE_WIDTH,
            RAW_IMAGE_HEIGHT,
            8,
            theMaskingImageURL,
            MASKING_IMAGE_WIDTH,
            MASKING_IMAGE_HEIGHT,
        )


def doClipMask(context):
    theMaskingImageURL = GetURL(kMaskingImage)

    if theMaskingImageURL is not None:
        ImageMasking.drawWithClippingMask(
            context, theMaskingImageURL, MASKING_IMAGE_WIDTH, MASKING_IMAGE_HEIGHT
        )


def tilePDFDocument(context, offscreenType):
    url = GetURL(kCatPDF)

    if url is not None:
        if offscreenType == noOffScreen:
            BitmapContext.TilePDFNoBuffer(context, url)
        elif offscreenType == bitmapOffScreen:
            BitmapContext.TilePDFWithOffscreenBitmap(context, url)
        else:
            BitmapContext.TilePDFWithCGLayer(context, url)


def doCompatibleEPSDrawing(context):
    ourEPSurl = GetURL(kOurEPS)

    if ourEPSurl is not None:
        EPSPrinting.drawEPSDataImage(context, ourEPSurl)


def DispatchDrawing(context, drawingType):
    """ Drawing dispatcher """
    if drawingType == UIHandling.kHICommandSimpleRect:
        DrawingBasics.doSimpleRect(context)

    elif drawingType == UIHandling.kHICommandStrokedRect:
        DrawingBasics.doStrokedRect(context)

    elif drawingType == UIHandling.kHICommandStrokedAndFilledRect:
        DrawingBasics.doStrokedAndFilledRect(context)

    elif drawingType == UIHandling.kHICommandPathRects:
        DrawingBasics.doPathRects(context)

    elif drawingType == UIHandling.kHICommandAlphaRects:
        DrawingBasics.doAlphaRects(context)

    elif drawingType == UIHandling.kHICommandDashed:
        DrawingBasics.doDashedLines(context)

    elif drawingType == UIHandling.kHICommandSimpleClip:
        DrawingBasics.doClippedCircle(context)

    elif drawingType == UIHandling.kHICommandPDFDoc:
        callPDFDrawProc(context, DrawingBasics.doPDFDocument, kCatPDF)

    elif drawingType == UIHandling.kHICommandRotatedEllipses:
        CoordinateSystem.doRotatedEllipses(context)

    elif drawingType == UIHandling.kHICommandDrawSkewCoordinates:
        CoordinateSystem.drawSkewedCoordinateSystem(context)

    elif drawingType == UIHandling.kHICommandBezierEgg:
        PathDrawing.doEgg(context)

    elif drawingType == UIHandling.kHICommandRoundedRects:
        PathDrawing.doRoundedRects(context)

    elif drawingType == UIHandling.kHICommandStrokeWithCTM:
        PathDrawing.doStrokeWithCTM(context)

    elif drawingType == UIHandling.kHICommandRotatedEllipsesWithCGPath:
        PathDrawing.doRotatedEllipsesWithCGPath(context)

    elif drawingType == UIHandling.kHICommandPixelAligned:
        PathDrawing.doPixelAlignedFillAndStroke(context)

    elif drawingType == UIHandling.kHICommandDeviceFillAndStrokeColor:
        ColorAndGState.doColorSpaceFillAndStroke(context)

    elif drawingType == UIHandling.kHICommandCLUTDrawGraphics:
        ColorAndGState.doIndexedColorDrawGraphics(context)

    elif drawingType == UIHandling.kHICommandDrawWithGlobalAlpha:
        ColorAndGState.drawWithGlobalAlpha(context)

    elif drawingType == UIHandling.kHICommandDrawWithBlendMode:
        callPDFDrawProc(
            context, ColorAndGState.drawWithColorBlendMode, kPDFForBlendMode
        )

    elif drawingType == UIHandling.kHICommandDrawWithColorRefs:
        ColorAndGState.drawWithColorRefs(context)

    elif drawingType == UIHandling.kHICommandFunctionsHaveOwnGSave:
        ColorAndGState.doClippedEllipse(context)

    elif drawingType == UIHandling.kHICommandDrawJPEGImage:
        doDrawJPEGFile(context)

    elif drawingType == UIHandling.kHICommandColorImageFromFile:
        doRawImageFileWithURL(context)

    elif drawingType == UIHandling.kHICommandColorImageFromData:
        Images.doColorRampImage(context)

    elif drawingType == UIHandling.kHICommandColorImageFromCallbacks:
        doRawImageFileWithCallbacks(context)

    elif drawingType == UIHandling.kHICommandGrayRamp:
        Images.doGrayRamp(context)

    elif drawingType == UIHandling.kHICommandDrawWithCGImageSource:
        doDrawImageWithCGImageSource(context)

    elif drawingType == UIHandling.kHICommandDrawWithCGImageSourceIncremental:
        doIncrementalImage(context)

    elif drawingType == UIHandling.kHICommandDrawWithQuickTime:
        doQTImage(context)

    elif drawingType == UIHandling.kHICommandSubstituteImageProfile:
        doJPEGDocumentWithMultipleProfiles(context)

    elif drawingType == UIHandling.kHICommandDoSubImage:
        Images.doColorRampSubImage(context)

    elif drawingType == UIHandling.kHICommandExportWithQuickTime:
        Images.exportColorRampImageWithQT(context)

    elif drawingType == UIHandling.kHICommandMaskTurkeyImage:
        ImageMasking.doOneBitMaskImages(context)

    elif drawingType == UIHandling.kHICommandImageMaskedWithMask:
        doMaskImageWithMask(context)

    elif drawingType == UIHandling.kHICommandImageMaskedWithGrayImage:
        doMaskImageWithGrayImage(context)

    elif drawingType == UIHandling.kHICommandMaskImageWithColor:
        doImageMaskedWithColor(context)

    elif drawingType == UIHandling.kHICommandClipToMask:
        doClipMask(context)

    elif drawingType == UIHandling.kHICommandExportWithCGImageDestination:
        exportImageMaskedWithImage(context)

    elif drawingType == UIHandling.kHICommandSimpleCGLayer:
        BitmapContext.doSimpleCGLayer(context)

    elif drawingType == UIHandling.kHICommandAlphaOnlyContext:
        BitmapContext.doAlphaOnlyContext(context)

    elif drawingType == UIHandling.kHICommandDrawNoOffScreenImage:
        tilePDFDocument(context, noOffScreen)

    elif drawingType == UIHandling.kHICommandDrawOffScreenImage:
        tilePDFDocument(context, bitmapOffScreen)

    elif drawingType == UIHandling.kHICommandDrawWithLayer:
        tilePDFDocument(context, layerOffScreen)

    elif drawingType == UIHandling.kHICommandQuartzRomanText:
        QuartzTextDrawing.drawQuartzRomanText(context)

    elif drawingType == UIHandling.kHICommandQuartzTextModes:
        QuartzTextDrawing.drawQuartzTextWithTextModes(context)

    elif drawingType == UIHandling.kHICommandQuartzTextMatrix:
        QuartzTextDrawing.drawQuartzTextWithTextMatrix(context)

    elif drawingType == UIHandling.kHICommandSimplePattern:
        PatternDrawing.doRedBlackCheckerboard(context)

    elif drawingType == UIHandling.kHICommandPatternPhase:
        PatternDrawing.doPatternPhase(context)

    elif drawingType == UIHandling.kHICommandPatternMatrix:
        PatternDrawing.doPatternMatrix(context)

    elif drawingType == UIHandling.kHICommandUncoloredPattern:
        PatternDrawing.doStencilPattern(context)

    elif drawingType == UIHandling.kHICommandDrawWithPDFPattern:
        callPDFDrawProc(context, PatternDrawing.drawWithPDFPattern, kCatPDF)

    elif drawingType == UIHandling.kHICommandSimpleShadow:
        ShadowsAndTransparencyLayers.drawSimpleShadow(context)

    elif drawingType == UIHandling.kHICommandShadowScaling:
        ShadowsAndTransparencyLayers.doShadowScaling(context)

    elif drawingType == UIHandling.kHICommandShadowProblems:
        ShadowsAndTransparencyLayers.showComplexShadowIssues(context)

    elif drawingType == UIHandling.kHICommandComplexShadow:
        ShadowsAndTransparencyLayers.showComplexShadow(context)

    elif drawingType == UIHandling.kHICommandMultipleShapeComposite:
        ShadowsAndTransparencyLayers.doLayerCompositing(context)

    elif drawingType == UIHandling.kHICommandFillAndStrokeWithShadow:
        ShadowsAndTransparencyLayers.drawFillAndStrokeWithShadow(context)

    elif drawingType == UIHandling.kHICommandPDFDocumentShadow:
        callPDFDrawProc(
            context, ShadowsAndTransparencyLayers.shadowPDFDocument, kCatPDF
        )

    elif drawingType == UIHandling.kHICommandSimpleAxialShading:
        Shadings.doSimpleAxialShading(context)

    elif drawingType == UIHandling.kHICommandExampleAxialShadings:
        Shadings.doExampleAxialShading(context)

    elif drawingType == UIHandling.kHICommandSimpleRadialShading:
        Shadings.doSimpleRadialShading(context)

    elif drawingType == UIHandling.kHICommandExampleRadialShadings:
        Shadings.doExampleRadialShadings(context)

    elif drawingType == UIHandling.kHICommandEllipseShading:
        Shadings.doEllipseShading(context)

    elif drawingType == UIHandling.kHICommandDoCompatibleEPS:
        doCompatibleEPSDrawing(context)



import Cocoa
import objc
import Quartz  # noqa: F401
from objc import super

# APPLICATION DATA STORAGE NOTES:
# - This application uses a simple data storage as an array of entries,
#   each containing two attributes: a label and a value
# - The array is represented as a NSMutableArray, the entries as
#   NSMutableDictionaries, the label as a NSString with the "label" key and
#   the value as a NSNumber with the "value" key
#
#
# QUARTZ COMPOSER COMPOSITION NOTES:
# - The enclosed Quartz Composer composition renders a 3D bars chart and is
#   loaded on the QCView in the application's window
# - This composition has three input parameters:
#   * "Data": the data to display by the chart which must be formatted as a
#     NSArray of NSDictionaries, each NSDictionary containing "label" / NSString
#     and "value" / NSNumber value-key pairs
#   * "Scale": a NSNumber used to scale the chart bars
#   * "Spacing": a NSNumber indicating the extra spacing between the chart bars
# - The "Data" and "Scale" input parameters are set programmatically while the
#   "Spacing" is set directly from the UI through Cocoa bindings
# - Note that this composition is quite simple and has the following
#   limitations:
#   * it may have rendering artifacts when looking at the chart from some angles
#   * it does not support negative values
#   * labels are not truncated if too long
# - Basically, the composition performs the following:
#   * renders a background gradient
#   * draws three planes on the X, Y and Z axes
#   * uses an Iterator patch to loop on the chart data, which is available as
#     a Structure, and for each member, retrieves the label and value, then
#     draws them
#   * the chart rendering is enclosed into a Camera macro patch used to center
#     it in the view
#   * the Camera macro patch is itself enclosed into a TrackBall macro patch
#     so that the user can rotate the chart with the mouse
#   * the TrackBall macro patch is itself enclosed into a Lighting macro patch
#     so that the chart is lighted
# - This composition makes uses of transparency for a nicer effect, but
#   neither OpenGL nor Quartz Composer handle automatically proper rendering
#   of mixed opaque and transparent 3D objects
# - A simple, but not fail-proof, algorithm to render opaque and transparent
#   3D objects is to:
#   * render opaque objects first with depth testing set to "Read / Write"
#   * render transparent objects with depth testing set to "Read-Only"
#
#
#  NIB FILES NOTES:
# - The QCView is configured to start rendering automatically and forward user
#   events (mouse events are required to rotate the chart)
# - An AppController instance is connected as the data source for the
#   NSTableView
# - The NSTableView is set up so that the identifiers of table columns match
#   the keys used in the data storage
# - The "Value" column of the NSTableView has a NSNumberFormatter which
#   guarantees only positive or null numbers can be entered here
# - The "Label" column of the NSTableView simply contains text
#

# Keys for the entries in the data storage
kDataKey_Label = "label"  # NSString
kDataKey_Value = "value"  # NSNumber

# Keys for the composition input parameters
kParameterKey_Data = "Data"  # NSArray of NSDictionaries
kParameterKey_Scale = "Scale"  # NSNumber
kParameterKey_Spacing = "Spacing"  # NSNumber


class AppController(Cocoa.NSObject):
    tableView = objc.IBOutlet()
    view = objc.IBOutlet()

    _data = objc.ivar()

    def init(self):
        # Allocate our data storage
        self = super(AppController, self).init()
        if self is None:
            return None

        self._data = []

        return self

    def awakeFromNib(self):
        # Load the composition file into the QCView (because this
        # QCView is bound to a QCPatchController in the nib file, this
        # will actually update the QCPatchController along with all the
        # bindings)
        if not self.view.loadCompositionFromFile_(
            Cocoa.NSBundle.mainBundle().pathForResource_ofType_("Chart", "qtz")
        ):
            Cocoa.NSLog("Composition loading failed")
            Cocoa.NSApp.terminate_(None)

        # Populate data storage
        self._data.extend(
            [
                {kDataKey_Label: "Palo Alto", kDataKey_Value: 2},
                {kDataKey_Label: "Cupertino", kDataKey_Value: 1},
                {kDataKey_Label: "Menlo Park", kDataKey_Value: 4},
                {kDataKey_Label: "Mountain View", kDataKey_Value: 8},
                {kDataKey_Label: "San Francisco", kDataKey_Value: 7},
                {kDataKey_Label: "Los Altos", kDataKey_Value: 3},
            ]
        )

        # Initialize the views
        self.tableView.reloadData()
        self.updateChart()

    def updateChart(self):
        # Update the data displayed by the chart - it will be converted to a
        # Structure of Structures by Quartz Composer
        self.view.setValue_forInputKey_(self._data, kParameterKey_Data)

        # Compute the maximum value and set the chart scale accordingly
        max_value = 0.0
        for obj in self._data:
            value = obj[kDataKey_Value]
            if value > max_value:
                max_value = value

        if max_value == 0.0:
            scale = 1.0
        else:
            scale = 1 / max_value
        self.view.setValue_forInputKey_(scale, kParameterKey_Scale)

    @objc.IBAction
    def addEntry_(self, sender):
        # Add a new entry to the data storage
        self._data.append({kDataKey_Label: "Untitled", kDataKey_Value: 0})

        # Notify the NSTableView and update the chart
        self.tableView.reloadData()
        self.updateChart()

        # Automatically select and edit the new entry
        self.tableView.selectRow_byExtendingSelection_(len(self._data) - 1, False)
        self.tableView.editColumn_row_withEvent_select_(
            self.tableView.columnWithIdentifier_(kDataKey_Label),
            len(self._data) - 1,
            None,
            True,
        )

    @objc.IBAction
    def removeEntry_(self, sender):
        # Make sure we have a valid selected row
        selectedRow = self.tableView.selectedRow()
        if selectedRow < 0 or self.tableView.editedRow() == selectedRow:
            return

        # Remove the currently selected entry from the data storage
        del self._data[selectedRow]

        # Notify the NSTableView and update the chart
        self.tableView.reloadData()
        self.updateChart()

    def numberOfRowsInTableView_(self, aTableView):
        # Return the number of entries in the data storage
        return len(self._data)

    def tableView_objectValueForTableColumn_row_(
        self, aTableView, aTableColumn, rowIndex
    ):
        # Get the "label" or "value" attribute of the entry from the data
        # storage at index "rowIndex"
        return self._data[rowIndex][aTableColumn.identifier()]

    def tableView_setObjectValue_forTableColumn_row_(
        self, aTableView, anObject, aTableColumn, rowIndex
    ):

        # Set the "label" or "value" attribute of the entry from the data
        # storage at index "rowIndex"
        self._data[rowIndex][aTableColumn.identifier()] = anObject

        # Update the chart
        self.updateChart()



import sys

import Cocoa
import Quartz

# Return the generic RGB color space. This is a 'get' function and the caller
# should not release the returned value unless the caller retains it first.
# Usually callers of this routine will immediately use the returned colorspace
# with CoreGraphics so they typically do not need to retain it themselves.
# This function creates the generic RGB color space once and hangs onto it so
# it can return it whenever this function is called.

_genericRGBColorSpace = None


def getTheCalibratedRGBColorSpace():
    global _genericRGBColorSpace

    if _genericRGBColorSpace is None:
        _genericRGBColorSpace = Quartz.CGColorSpaceCreateWithName(
            Quartz.kCGColorSpaceGenericRGB
        )

    return _genericRGBColorSpace


def addURLAnnotationToPDFPage(c, rect):
    link = "http:#developer.apple.com/documentation/GraphicsImaging/"
    linkURL = Cocoa.CFURLCreateWithBytes(
        None, link, len(link), Cocoa.kCFStringEncodingUTF8, None
    )
    if linkURL is None:
        print("Couldn't create url for link!")
        return

    Quartz.CGPDFContextSetURLForRect(c, linkURL, rect)


def myCreatePDFDocumentAtURL(url):
    red = [1.0, 0.0, 0.0, 1.0]

    # Make the media box the same size as the graphics this code draws.
    mediaBox = Quartz.CGRectMake(0, 0, 200, 200)

    # Create a PDF context using mediaBox as the default media box for the
    # document. The document will be created at the location described by
    # the supplied URL. This example passes None for the auxiliaryInfo
    # dictionary.
    pdfContext = Quartz.CGPDFContextCreateWithURL(url, mediaBox, None)
    if pdfContext is None:
        print("Couldn't create PDF context!")
        return

    # Calling CGContextBeginPage indicates the following content is
    # to appear on the first page. The rect passed to this function
    # specifies the media box for this page.
    mediaBox = Quartz.CGContextBeginPage(pdfContext, None)
    # It is good programming practice to bracket the drawing
    # you do on a page with saving and restoring the graphics
    # state.
    Quartz.CGContextSaveGState(pdfContext)
    if 1:
        # Clip to the media box.
        Quartz.CGContextClipToRect(pdfContext, mediaBox)
        # Set the fill color and color space.
        Quartz.CGContextSetFillColorSpace(pdfContext, getTheCalibratedRGBColorSpace())
        Quartz.CGContextSetFillColor(pdfContext, red)
        # Fill the rectangle of the media box with red.
        Quartz.CGContextFillRect(pdfContext, mediaBox)

        # Add a link to a URL so that if you click
        # on the rect you will go to that URL.
        addURLAnnotationToPDFPage(pdfContext, mediaBox)
    Quartz.CGContextRestoreGState(pdfContext)

    # Calling CGContextEndPage denotes the end of the first page.
    # You MUST call CGContextEndPage after each time you call
    # CGContextBeginPage and they should not be nested.
    Quartz.CGContextEndPage(pdfContext)

    # You MUST release the PDF context when done with it. When the
    # retain count on the context reaches zero, Quartz flushes the
    # drawing content to the PDF file being created and closes it.
    Quartz.CGContextRelease(pdfContext)


# This function returns a None CFStringRef if the password
# is greater than 32 bytes long or the string contains
# characters outside the range 32-127 inclusive.
def createPasswordString(password):
    if not isinstance(password, bytes):
        password = password.encode("utf-8")

    # Check the length.
    if len(password) > 32:
        return None

    # Check that the byte codes are in the printable ASCII range.
    for ch in password:
        if not (32 <= (ord(ch) if sys.version_info[0] == 2 else ch) <= 127):
            return None

    return Cocoa.CFStringCreateWithCString(None, password, Cocoa.kCFStringEncodingASCII)


def addEncryptionKeys(info_dict):
    ownerPassword = "test"
    ownerPasswordRef = None
    if info_dict is None:
        return

    ownerPasswordRef = createPasswordString(ownerPassword)
    if ownerPasswordRef is None:
        print("Invalid owner password %s!" % (ownerPassword,))
        return

    if hasattr(Quartz, "kCGPDFContextOwnerPassword"):
        # Add the owner password.
        info_dict[Quartz.kCGPDFContextOwnerPassword] = ownerPasswordRef

        # No user password supplied so Quartz will use the empty string.

        # Mark that printing is disallowed.
        info_dict[Quartz.kCGPDFContextAllowsPrinting] = False

    else:
        print("Encrypted PDF not available in this version of macOS!")


def myCreate2PagePDFDocumentAtURL(url):
    red = [1.0, 0.0, 0.0, 1.0]
    blue = [0.0, 0.0, 1.0, 1.0]
    redPageName = "com.mycompany.links.dg.redpage"
    bluePageName = "com.mycompany.links.dg.bluepage"

    # Make the media box the same size as a US Letter size page.
    mediaBox = Quartz.CGRectMake(0, 0, 612, 792)
    rectBox = Quartz.CGRectMake(55, 55, 500, 680)
    # Create a point whose center is the center of rectBox.
    centerPoint = Quartz.CGPoint(
        rectBox.origin.x + rectBox.size.width / 2,
        rectBox.origin.y + rectBox.size.height / 2,
    )

    auxiliaryInfo = {}
    addEncryptionKeys(auxiliaryInfo)
    pdfContext = Quartz.CGPDFContextCreateWithURL(url, mediaBox, auxiliaryInfo)

    if pdfContext is None:
        print("Couldn't create PDF context!")
        return

    # Start the first page
    Quartz.CGContextBeginPage(pdfContext, mediaBox)
    Quartz.CGContextSaveGState(pdfContext)
    if 1:
        # Clip to the media box.
        Quartz.CGContextClipToRect(pdfContext, mediaBox)
        # Set the fill color and color space.
        Quartz.CGContextSetFillColorSpace(pdfContext, getTheCalibratedRGBColorSpace())
        Quartz.CGContextSetFillColor(pdfContext, red)
        # Fill the rectangle of the media box with red.
        Quartz.CGContextFillRect(pdfContext, rectBox)
        # Make a new named destination at the center of the rect being
        # painted. Here the code uses the name redPageName since
        # this is the "red" page that is being named.
        Quartz.CGPDFContextAddDestinationAtPoint(pdfContext, redPageName, centerPoint)
        # Make a link to a destination not yet created, that for
        # the "blue" page. Making this link is independent from
        # the creation of the destination above. Clicking
        # on this link in the generated PDF document navigates to
        # the destination referenced by bluePageName.
        Quartz.CGPDFContextSetDestinationForRect(pdfContext, bluePageName, rectBox)
    Quartz.CGContextRestoreGState(pdfContext)
    Quartz.CGContextEndPage(pdfContext)

    # Start the second page.
    Quartz.CGContextBeginPage(pdfContext, mediaBox)
    Quartz.CGContextSaveGState(pdfContext)
    if 1:
        # Clip to the media box.
        Quartz.CGContextClipToRect(pdfContext, mediaBox)
        # Set the fill color and color space.
        Quartz.CGContextSetFillColorSpace(pdfContext, getTheCalibratedRGBColorSpace())
        Quartz.CGContextSetFillColor(pdfContext, blue)
        # Fill the rectangle of the media box with blue.
        Quartz.CGContextFillRect(pdfContext, rectBox)
        # Make a new named destination at the center of the rect
        # being painted. Here the code uses the name bluePageName
        # since this is the "blue" page that is being named. The link
        # on page 1 refers to this destination.
        Quartz.CGPDFContextAddDestinationAtPoint(pdfContext, bluePageName, centerPoint)
        # Make a link to a destination already created
        # for page 1, the red page. Clicking on this link
        # in the generated PDF document navigates to
        # the destination referenced by redPageName.
        Quartz.CGPDFContextSetDestinationForRect(pdfContext, redPageName, rectBox)
    Quartz.CGContextRestoreGState(pdfContext)
    Quartz.CGContextEndPage(pdfContext)

    # You MUST release the PDF context when done with it. When the
    # retain count on the context reaches zero, Quartz flushes the
    # drawing content to the PDF file being created and closes it.
    del pdfContext


def main(args=None):
    if args is None:
        args = sys.argv

    path = b"output.pdf"
    url = Cocoa.CFURLCreateFromFileSystemRepresentation(None, path, len(path), False)
    if url is None:
        print("Couldn't create URL!")
        return 1

    myCreate2PagePDFDocumentAtURL(url)
    return 0


if __name__ == "__main__":
    sys.exit(main())



# This file is generated by objective.metadata
#
# Last update: Sun Aug  4 21:30:28 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$IKFilterBrowserDefaultInputImage$IKFilterBrowserExcludeCategories$IKFilterBrowserExcludeFilters$IKFilterBrowserFilterDoubleClickNotification$IKFilterBrowserFilterSelectedNotification$IKFilterBrowserShowCategories$IKFilterBrowserShowPreview$IKFilterBrowserWillPreviewFilterNotification$IKImageBrowserBackgroundColorKey$IKImageBrowserCGImageRepresentationType$IKImageBrowserCGImageSourceRepresentationType$IKImageBrowserCellBackgroundLayer$IKImageBrowserCellForegroundLayer$IKImageBrowserCellLayerTypeBackground$IKImageBrowserCellLayerTypeForeground$IKImageBrowserCellLayerTypePlaceHolder$IKImageBrowserCellLayerTypeSelection$IKImageBrowserCellPlaceHolderLayer$IKImageBrowserCellSelectionLayer$IKImageBrowserCellsHighlightedTitleAttributesKey$IKImageBrowserCellsOutlineColorKey$IKImageBrowserCellsSubtitleAttributesKey$IKImageBrowserCellsTitleAttributesKey$IKImageBrowserGroupBackgroundColorKey$IKImageBrowserGroupFooterLayer$IKImageBrowserGroupHeaderLayer$IKImageBrowserGroupRangeKey$IKImageBrowserGroupStyleKey$IKImageBrowserGroupTitleKey$IKImageBrowserIconRefPathRepresentationType$IKImageBrowserIconRefRepresentationType$IKImageBrowserNSBitmapImageRepresentationType$IKImageBrowserNSDataRepresentationType$IKImageBrowserNSImageRepresentationType$IKImageBrowserNSURLRepresentationType$IKImageBrowserPDFPageRepresentationType$IKImageBrowserPathRepresentationType$IKImageBrowserQCCompositionPathRepresentationType$IKImageBrowserQCCompositionRepresentationType$IKImageBrowserQTMoviePathRepresentationType$IKImageBrowserQTMovieRepresentationType$IKImageBrowserQuickLookPathRepresentationType$IKImageBrowserSelectionColorKey$IKOverlayTypeBackground$IKOverlayTypeImage$IKPictureTakerAllowsEditingKey$IKPictureTakerAllowsFileChoosingKey$IKPictureTakerAllowsVideoCaptureKey$IKPictureTakerCropAreaSizeKey$IKPictureTakerImageTransformsKey$IKPictureTakerInformationalTextKey$IKPictureTakerOutputImageMaxSizeKey$IKPictureTakerRemainOpenAfterValidateKey$IKPictureTakerShowAddressBookPicture$IKPictureTakerShowAddressBookPictureKey$IKPictureTakerShowEffectsKey$IKPictureTakerShowEmptyPicture$IKPictureTakerShowEmptyPictureKey$IKPictureTakerShowRecentPictureKey$IKPictureTakerUpdateRecentPictureKey$IKSlideshowAudioFile$IKSlideshowModeImages$IKSlideshowModeOther$IKSlideshowModePDF$IKSlideshowPDFDisplayBox$IKSlideshowPDFDisplayMode$IKSlideshowPDFDisplaysAsBook$IKSlideshowScreen$IKSlideshowStartIndex$IKSlideshowStartPaused$IKSlideshowWrapAround$IKToolModeAnnotate$IKToolModeCrop$IKToolModeMove$IKToolModeNone$IKToolModeRotate$IKToolModeSelect$IKToolModeSelectEllipse$IKToolModeSelectLasso$IKToolModeSelectRect$IKUIFlavorAllowFallback$IKUISizeFlavor$IKUISizeMini$IKUISizeRegular$IKUISizeSmall$IKUImaxSize$IK_ApertureBundleIdentifier$IK_MailBundleIdentifier$IK_PhotosBundleIdentifier$IK_iPhotoBundleIdentifier$"""
enums = """$IKCameraDeviceViewDisplayModeIcon@1$IKCameraDeviceViewDisplayModeNone@-1$IKCameraDeviceViewDisplayModeTable@0$IKCameraDeviceViewTransferModeFileBased@0$IKCameraDeviceViewTransferModeMemoryBased@1$IKCellsStyleNone@0$IKCellsStyleOutlined@2$IKCellsStyleShadowed@1$IKCellsStyleSubtitled@8$IKCellsStyleTitled@4$IKDeviceBrowserViewDisplayModeIcon@2$IKDeviceBrowserViewDisplayModeOutline@1$IKDeviceBrowserViewDisplayModeTable@0$IKGroupBezelStyle@0$IKGroupDisclosureStyle@1$IKImageBrowserDropBefore@1$IKImageBrowserDropOn@0$IKImageStateInvalid@1$IKImageStateNoImage@0$IKImageStateReady@2$IKScannerDeviceViewDisplayModeAdvanced@1$IKScannerDeviceViewDisplayModeNone@-1$IKScannerDeviceViewDisplayModeSimple@0$IKScannerDeviceViewTransferModeFileBased@0$IKScannerDeviceViewTransferModeMemoryBased@1$"""
misc.update({})
aliases = {
    "IKImagePickerShowEffectsKey": "IKPictureTakerShowEffectsKey",
    "IKImagePickerOutputImageMaxSizeKey": "IKPictureTakerOutputImageMaxSizeKey",
    "IKImagePickerImageTransformsKey": "IKPictureTakerImageTransformsKey",
    "IKImagePickerAllowsFileChoosingKey": "IKPictureTakerAllowsFileChoosingKey",
    "IK_API_DEPRECATED": "API_DEPRECATED",
    "IKImagePickerAllowsEditingKey": "IKPictureTakerAllowsEditingKey",
    "IKImagePickerInformationalTextKey": "IKPictureTakerInformationalTextKey",
    "IKImagePickerCropAreaSizeKey": "IKPictureTakerCropAreaSizeKey",
    "IKImagePickerAllowsVideoCaptureKey": "IKPictureTakerAllowsVideoCaptureKey",
    "IKImagePickerUpdateRecentPictureKey": "IKPictureTakerUpdateRecentPictureKey",
    "IKImagePickerShowRecentPictureKey": "IKPictureTakerShowRecentPictureKey",
}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"IKCameraDeviceView", b"canDeleteSelectedItems", {"retval": {"type": b"Z"}})
    r(b"IKCameraDeviceView", b"canDownloadSelectedItems", {"retval": {"type": b"Z"}})
    r(b"IKCameraDeviceView", b"canRotateSelectedItemsLeft", {"retval": {"type": b"Z"}})
    r(b"IKCameraDeviceView", b"canRotateSelectedItemsRight", {"retval": {"type": b"Z"}})
    r(
        b"IKCameraDeviceView",
        b"displaysDownloadsDirectoryControl",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"IKCameraDeviceView",
        b"displaysPostProcessApplicationControl",
        {"retval": {"type": b"Z"}},
    )
    r(b"IKCameraDeviceView", b"hasDisplayModeIcon", {"retval": {"type": b"Z"}})
    r(b"IKCameraDeviceView", b"hasDisplayModeTable", {"retval": {"type": b"Z"}})
    r(
        b"IKCameraDeviceView",
        b"selectIndexes:byExtendingSelection:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"IKCameraDeviceView",
        b"setDisplaysDownloadsDirectoryControl:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKCameraDeviceView",
        b"setDisplaysPostProcessApplicationControl:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKCameraDeviceView",
        b"setHasDisplayModeIcon:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKCameraDeviceView",
        b"setHasDisplayModeTable:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"IKDeviceBrowserView", b"displaysLocalCameras", {"retval": {"type": b"Z"}})
    r(b"IKDeviceBrowserView", b"displaysLocalScanners", {"retval": {"type": b"Z"}})
    r(b"IKDeviceBrowserView", b"displaysNetworkCameras", {"retval": {"type": b"Z"}})
    r(b"IKDeviceBrowserView", b"displaysNetworkScanners", {"retval": {"type": b"Z"}})
    r(
        b"IKDeviceBrowserView",
        b"setDisplaysLocalCameras:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKDeviceBrowserView",
        b"setDisplaysLocalScanners:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKDeviceBrowserView",
        b"setDisplaysNetworkCameras:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKDeviceBrowserView",
        b"setDisplaysNetworkScanners:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKFilterBrowserPanel",
        b"beginSheetWithOptions:modalForWindow:modalDelegate:didEndSelector:contextInfo:",
        {"arguments": {5: {"sel_of_type": sel32or64(b"v@:@i^v", b"v@:@q^v")}}},
    )
    r(
        b"IKFilterBrowserPanel",
        b"beginWithOptions:modelessDelegate:didEndSelector:contextInfo:",
        {"arguments": {4: {"sel_of_type": sel32or64(b"v@:@i^v", b"v@:@q^v")}}},
    )
    r(b"IKFilterBrowserView", b"setPreviewState:", {"arguments": {2: {"type": b"Z"}}})
    r(b"IKImageBrowserCell", b"isSelected", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"allowsDroppingOnItems", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"allowsEmptySelection", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"allowsMultipleSelection", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"allowsReordering", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"animates", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"canControlQuickLookPanel", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"constrainsToOriginalSize", {"retval": {"type": b"Z"}})
    r(b"IKImageBrowserView", b"isGroupExpandedAtIndex:", {"retval": {"type": b"Z"}})
    r(
        b"IKImageBrowserView",
        b"setAllowsDroppingOnItems:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKImageBrowserView",
        b"setAllowsEmptySelection:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKImageBrowserView",
        b"setAllowsMultipleSelection:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKImageBrowserView",
        b"setAllowsReordering:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"IKImageBrowserView", b"setAnimates:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"IKImageBrowserView",
        b"setCanControlQuickLookPanel:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKImageBrowserView",
        b"setConstrainsToOriginalSize:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKImageBrowserView",
        b"setSelectionIndexes:byExtendingSelection:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"IKImagePicker",
        b"beginImagePickerSheetForWindow:withDelegate:didEndSelector:contextInfo:",
        {"arguments": {4: {"sel_of_type": sel32or64(b"v@:@I^v", b"v@:@Q^v")}}},
    )
    r(
        b"IKImagePicker",
        b"beginImagePickerWithDelegate:didEndSelector:contextInfo:",
        {"arguments": {3: {"sel_of_type": sel32or64(b"v@:@I^v", b"v@:@Q^v")}}},
    )
    r(b"IKImageView", b"autohidesScrollers", {"retval": {"type": b"Z"}})
    r(b"IKImageView", b"autoresizes", {"retval": {"type": b"Z"}})
    r(b"IKImageView", b"doubleClickOpensImageEditPanel", {"retval": {"type": b"Z"}})
    r(b"IKImageView", b"editable", {"retval": {"type": b"Z"}})
    r(b"IKImageView", b"hasHorizontalScroller", {"retval": {"type": b"Z"}})
    r(b"IKImageView", b"hasVerticalScroller", {"retval": {"type": b"Z"}})
    r(b"IKImageView", b"setAutohidesScrollers:", {"arguments": {2: {"type": b"Z"}}})
    r(b"IKImageView", b"setAutoresizes:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"IKImageView",
        b"setDoubleClickOpensImageEditPanel:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"IKImageView", b"setEditable:", {"arguments": {2: {"type": b"Z"}}})
    r(b"IKImageView", b"setHasHorizontalScroller:", {"arguments": {2: {"type": b"Z"}}})
    r(b"IKImageView", b"setHasVerticalScroller:", {"arguments": {2: {"type": b"Z"}}})
    r(b"IKImageView", b"setSupportsDragAndDrop:", {"arguments": {2: {"type": b"Z"}}})
    r(b"IKImageView", b"supportsDragAndDrop", {"retval": {"type": b"Z"}})
    r(
        b"IKPictureTaker",
        b"beginPictureTakerSheetForWindow:withDelegate:didEndSelector:contextInfo:",
        {"arguments": {4: {"sel_of_type": sel32or64(b"v@:@i^v", b"v@:@q^v")}}},
    )
    r(
        b"IKPictureTaker",
        b"beginPictureTakerWithDelegate:didEndSelector:contextInfo:",
        {"arguments": {3: {"sel_of_type": sel32or64(b"v@:@i^v", b"v@:@q^v")}}},
    )
    r(b"IKPictureTaker", b"mirroring", {"retval": {"type": b"Z"}})
    r(
        b"IKPictureTaker",
        b"popUpRecentsMenuForView:withDelegate:didEndSelector:contextInfo:",
        {"arguments": {4: {"sel_of_type": sel32or64(b"v@:@i^v", b"v@:@q^v")}}},
    )
    r(b"IKPictureTaker", b"setMirroring:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"IKScannerDeviceView",
        b"displaysDownloadsDirectoryControl",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"IKScannerDeviceView",
        b"displaysPostProcessApplicationControl",
        {"retval": {"type": b"Z"}},
    )
    r(b"IKScannerDeviceView", b"hasDisplayModeAdvanced", {"retval": {"type": b"Z"}})
    r(b"IKScannerDeviceView", b"hasDisplayModeSimple", {"retval": {"type": b"Z"}})
    r(
        b"IKScannerDeviceView",
        b"setDisplaysDownloadsDirectoryControl:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKScannerDeviceView",
        b"setDisplaysPostProcessApplicationControl:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKScannerDeviceView",
        b"setHasDisplayModeAdvanced:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(
        b"IKScannerDeviceView",
        b"setHasDisplayModeSimple:",
        {"arguments": {2: {"type": b"Z"}}},
    )
    r(b"IKSlideshow", b"canExportToApplication:", {"retval": {"type": b"Z"}})
    r(
        b"NSObject",
        b"cameraDeviceView:didDownloadFile:location:fileData:error:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": b"@"},
                6: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"cameraDeviceView:didEncounterError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"cameraDeviceViewSelectionDidChange:",
        {"required": False, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"canExportSlideshowItemAtIndex:toApplication:",
        {
            "required": False,
            "retval": {"type": b"Z"},
            "arguments": {2: {"type": sel32or64(b"I", b"Q")}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"deviceBrowserView:didEncounterError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"deviceBrowserView:selectionDidChange:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(b"NSObject", b"hasAdjustMode", {"required": False, "retval": {"type": b"Z"}})
    r(b"NSObject", b"hasDetailsMode", {"required": False, "retval": {"type": b"Z"}})
    r(b"NSObject", b"hasEffectsMode", {"required": False, "retval": {"type": b"Z"}})
    r(b"NSObject", b"image", {"required": True, "retval": {"type": b"^{CGImage=}"}})
    r(
        b"NSObject",
        b"imageBrowser:backgroundWasRightClickedWithEvent:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"imageBrowser:cellAtIndex:",
        {
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"I", b"L")}},
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:cellWasDoubleClickedAtIndex:",
        {
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:cellWasRightClickedAtIndex:withEvent:",
        {
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": sel32or64(b"I", b"Q")},
                4: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:groupAtIndex:",
        {
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:itemAtIndex:",
        {
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:moveCellsAtIndexes:toIndex:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": sel32or64(b"I", b"L")},
            },
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:moveItemsAtIndexes:toIndex:",
        {
            "retval": {"type": b"Z"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": sel32or64(b"I", b"Q")},
            },
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:removeCellsAtIndexes:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"imageBrowser:removeItemsAtIndexes:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"imageBrowser:writeCellsAtIndexes:toPasteboard:",
        {
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"imageBrowser:writeItemsAtIndexes:toPasteboard:",
        {
            "retval": {"type": sel32or64(b"I", b"Q")},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"imageBrowserSelectionDidChange:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"imageProperties", {"required": False, "retval": {"type": b"@"}})
    r(b"NSObject", b"imageRepresentation", {"retval": {"type": b"@"}})
    r(b"NSObject", b"imageRepresentationType", {"retval": {"type": b"@"}})
    r(b"NSObject", b"imageSubtitle", {"retval": {"type": b"@"}})
    r(b"NSObject", b"imageTitle", {"retval": {"type": b"@"}})
    r(b"NSObject", b"imageUID", {"retval": {"type": b"@"}})
    r(b"NSObject", b"imageVersion", {"retval": {"type": sel32or64(b"I", b"Q")}})
    r(b"NSObject", b"isSelectable", {"retval": {"type": b"Z"}})
    r(
        b"NSObject",
        b"nameOfSlideshowItemAtIndex:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"numberOfCellsInImageBrowser:",
        {"retval": {"type": sel32or64(b"I", b"L")}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"numberOfGroupsInImageBrowser:",
        {"retval": {"type": sel32or64(b"I", b"Q")}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"numberOfItemsInImageBrowser:",
        {"retval": {"type": sel32or64(b"I", b"Q")}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"numberOfSlideshowItems",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"provideViewForUIConfiguration:excludedKeys:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"saveOptions:shouldShowUTType:",
        {"retval": {"type": b"Z"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"scannerDeviceView:didEncounterError:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"scannerDeviceView:didScanToBandData:scanInfo:error:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"scannerDeviceView:didScanToURL:error:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"scannerDeviceView:didScanToURL:fileData:error:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": b"@"},
                5: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"setImage:imageProperties:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"^{CGImage=}"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"slideshowDidChangeCurrentIndex:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(b"NSObject", b"slideshowDidStop", {"required": False, "retval": {"type": b"v"}})
    r(
        b"NSObject",
        b"slideshowItemAtIndex:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(b"NSObject", b"slideshowWillStart", {"required": False, "retval": {"type": b"v"}})
    r(
        b"NSObject",
        b"thumbnailWithMaximumSize:",
        {
            "required": False,
            "retval": {"type": b"^{CGImage=}"},
            "arguments": {2: {"type": sel32or64(b"{_NSSize=ff}", b"{CGSize=dd}")}},
        },
    )
    r(b"null", b"saveOptions:shouldShowUTType:", {"retval": {"type": b"Z"}})
finally:
    objc._updatingMetadata(False)
protocols = {
    "IKImageBrowserItem": objc.informal_protocol(
        "IKImageBrowserItem",
        [
            objc.selector(None, b"imageTitle", b"@@:", isRequired=False),
            objc.selector(None, b"imageSubtitle", b"@@:", isRequired=False),
            objc.selector(None, b"imageRepresentationType", b"@@:", isRequired=False),
            objc.selector(None, b"imageUID", b"@@:", isRequired=False),
            objc.selector(None, b"isSelectable", b"Z@:", isRequired=False),
            objc.selector(
                None, b"imageVersion", sel32or64(b"I@:", b"Q@:"), isRequired=False
            ),
            objc.selector(None, b"imageRepresentation", b"@@:", isRequired=False),
        ],
    ),
    "IKImageBrowserDataSourceDeprecated": objc.informal_protocol(
        "IKImageBrowserDataSourceDeprecated",
        [
            objc.selector(
                None,
                b"imageBrowser:moveCellsAtIndexes:toIndex:",
                sel32or64(b"Z@:@@I", b"Z@:@@L"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"imageBrowser:cellAtIndex:",
                sel32or64(b"@@:@I", b"@@:@L"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"numberOfCellsInImageBrowser:",
                sel32or64(b"I@:@", b"L@:@"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"imageBrowser:writeCellsAtIndexes:toPasteboard:",
                b"v@:@@@",
                isRequired=False,
            ),
            objc.selector(
                None, b"imageBrowser:removeCellsAtIndexes:", b"v@:@@", isRequired=False
            ),
        ],
    ),
    "IKSaveOptionsDelegate": objc.informal_protocol(
        "IKSaveOptionsDelegate",
        [
            objc.selector(
                None, b"saveOptions:shouldShowUTType:", b"Z@:@@", isRequired=False
            )
        ],
    ),
    "IKImageBrowserDelegate": objc.informal_protocol(
        "IKImageBrowserDelegate",
        [
            objc.selector(
                None,
                b"imageBrowser:cellWasRightClickedAtIndex:withEvent:",
                sel32or64(b"v@:@I@", b"v@:@Q@"),
                isRequired=False,
            ),
            objc.selector(
                None, b"imageBrowserSelectionDidChange:", b"v@:@", isRequired=False
            ),
            objc.selector(
                None,
                b"imageBrowser:cellWasDoubleClickedAtIndex:",
                sel32or64(b"v@:@I", b"v@:@Q"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"imageBrowser:backgroundWasRightClickedWithEvent:",
                b"v@:@@",
                isRequired=False,
            ),
        ],
    ),
    "IKImageBrowserDataSource": objc.informal_protocol(
        "IKImageBrowserDataSource",
        [
            objc.selector(
                None,
                b"imageBrowser:groupAtIndex:",
                sel32or64(b"@@:@I", b"@@:@Q"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"numberOfItemsInImageBrowser:",
                sel32or64(b"I@:@", b"Q@:@"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"imageBrowser:moveItemsAtIndexes:toIndex:",
                sel32or64(b"Z@:@@I", b"Z@:@@Q"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"numberOfGroupsInImageBrowser:",
                sel32or64(b"I@:@", b"Q@:@"),
                isRequired=False,
            ),
            objc.selector(
                None,
                b"imageBrowser:itemAtIndex:",
                sel32or64(b"@@:@I", b"@@:@Q"),
                isRequired=False,
            ),
            objc.selector(
                None, b"imageBrowser:removeItemsAtIndexes:", b"v@:@@", isRequired=False
            ),
            objc.selector(
                None,
                b"imageBrowser:writeItemsAtIndexes:toPasteboard:",
                sel32or64(b"I@:@@@", b"Q@:@@@"),
                isRequired=False,
            ),
        ],
    ),
}
expressions = {}

# END OF FILE



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

setup(
    name="PyChart",
    app=["main.py"],
    data_files=["English.lproj", "Chart.qtz"],
    setup_requires=["py2app", "pyobjc-framework-Cocoa", "pyobjc-framework-Quartz"],
)



"""
Add a watermark to all pages in a PDF document
"""
import math
import os
import sys

import Cocoa
import Quartz


def usage(name):
    print("Usage %s [inputfile]" % (name,))


class MyPDFData(object):
    pdfDoc = None
    mediaRect = None


# This is a simple function to create a CFURLRef from
# a path to a file. The path can be relative to the
# current directory or an absolute path.
def createURL(path):
    if not isinstance(path, bytes):
        path = path.encode("utf-8")
    return Cocoa.CFURLCreateFromFileSystemRepresentation(None, path, len(path), False)


# For the supplied URL and media box, create a PDF context
# that creates a PDF file at that URL and uses supplied rect
# as its document media box.
def myCreatePDFContext(url, mediaBox):
    options = {}
    options[Quartz.kCGPDFContextCreator] = "PDF Stamper Application"

    pdfContext = Quartz.CGPDFContextCreateWithURL(url, mediaBox, options)
    return pdfContext


# For a URL corresponding to an existing PDF document on disk,
# create a CGPDFDocumentRef and obtain the media box of the first
# page.
def myCreatePDFSourceDocument(url):
    myPDFData = MyPDFData()
    myPDFData.pdfDoc = Quartz.CGPDFDocumentCreateWithURL(url)
    if myPDFData.pdfDoc is not None:
        # NOTE: the original code uses CGPDFDocumentGetMediaBox, but that
        # API is deprecated and doesn't work in Leopard.
        page = Quartz.CGPDFDocumentGetPage(myPDFData.pdfDoc, 1)
        myPDFData.mediaRect = Quartz.CGPDFPageGetBoxRect(page, Quartz.kCGPDFMediaBox)

        # Make the media rect origin at 0,0.
        myPDFData.mediaRect.origin.x = myPDFData.mediaRect.origin.y = 0.0

    return myPDFData


# Draw the source PDF document into the context and then draw the stamp PDF document
# on top of it. When drawing the stamp on top, place it along the diagonal from the lower
# left corner to the upper right corner and center its media rect to the center of that
# diagonal.
def StampWithPDFDocument(context, sourcePDFDoc, stampFileDoc, stampMediaRect):
    numPages = Quartz.CGPDFDocumentGetNumberOfPages(sourcePDFDoc)

    # Loop over document pages and stamp each one appropriately.
    for i in range(1, numPages + 1):
        # Use the page rectangle of each page from the source to compute
        # the destination media box for each page and the location of
        # the stamp.

        # NOTE: the original code uses CGPDFDocumentGetMediaBox, but that
        # API is deprecated and doesn't work in Leopard.
        page = Quartz.CGPDFDocumentGetPage(sourcePDFDoc, i)
        pageRect = Quartz.CGPDFPageGetBoxRect(page, Quartz.kCGPDFMediaBox)

        Quartz.CGContextBeginPage(context, pageRect)
        Quartz.CGContextSaveGState(context)
        # Clip to the media box of the page.
        Quartz.CGContextClipToRect(context, pageRect)
        # First draw the content of the source document.
        Quartz.CGContextDrawPDFDocument(context, pageRect, sourcePDFDoc, i)
        # Translate to center of destination rect, that is the center of
        # the media box of content to draw on top of.
        Quartz.CGContextTranslateCTM(
            context, pageRect.size.width / 2, pageRect.size.height / 2
        )
        # Compute angle of the diagonal across the destination page.
        angle = math.atan(pageRect.size.height / pageRect.size.width)
        # Rotate by an amount so that drawn content goes along a diagonal
        # axis across the page.
        Quartz.CGContextRotateCTM(context, angle)
        # Move the origin so that the media box of the PDF to stamp
        # is centered around center point of destination.
        Quartz.CGContextTranslateCTM(
            context, -stampMediaRect.size.width / 2, -stampMediaRect.size.height / 2
        )
        # Now draw the document to stamp with on top of original content.
        Quartz.CGContextDrawPDFDocument(context, stampMediaRect, stampFileDoc, 1)
        Quartz.CGContextRestoreGState(context)
        Quartz.CGContextEndPage(context)


# From an input PDF document and a PDF document whose contents you
# want to draw on top of the other, create a new PDF document
# containing all the pages of the input document with the first page
# of the "stamping" overlayed.
def createStampedFileWithFile(inURL, stampURL, outURL):
    sourceFileData = myCreatePDFSourceDocument(inURL)
    if sourceFileData.pdfDoc is None:
        print("Can't create PDFDocumentRef for source input file!")
        return

    stampFileData = myCreatePDFSourceDocument(stampURL)
    if stampFileData.pdfDoc is None:
        Quartz.CGPDFDocumentRelease(sourceFileData.pdfDoc)
        print("Can't create PDFDocumentRef for file to stamp with!")
        return

    pdfContext = myCreatePDFContext(outURL, sourceFileData.mediaRect)
    if pdfContext is None:
        print("Can't create PDFContext for output file!")
        return

    StampWithPDFDocument(
        pdfContext, sourceFileData.pdfDoc, stampFileData.pdfDoc, stampFileData.mediaRect
    )


def main(args=None):
    if args is None:
        args = sys.argv

    suffix = ".watermarked.pdf"
    stampFileName = os.path.join(os.path.dirname(__file__), "confidential.pdf")

    if len(args) != 2:
        usage(args[0])
        return 1

    inputFileName = args[1]
    outputFileName = os.path.splitext(inputFileName)[0] + suffix

    inURL = createURL(inputFileName)
    if inURL is None:
        print("Couldn't create URL for input file!")
        return 1

    outURL = createURL(outputFileName)
    if outURL is None:
        print("Couldn't create URL for output file!")
        return 1

    stampURL = createURL(stampFileName)
    if stampURL is None:
        print("Couldn't create URL for stamping file!")
        return 1

    createStampedFileWithFile(inURL, stampURL, outURL)
    return 0


if __name__ == "__main__":
    sys.exit(main())



import os
import sys

import Cocoa
import objc
import Quartz


def createDataProviderFromPathName(path):
    # Create a CFURL for the supplied file system path.
    url = Cocoa.CFURLCreateWithFileSystemPath(
        None, path, Cocoa.kCFURLPOSIXPathStyle, False
    )
    if url is None:
        print("Couldn't create url!")
        return None

    # Create a Quartz data provider for the URL.
    dataProvider = Quartz.CGDataProviderCreateWithURL(url)
    if dataProvider is None:
        print("Couldn't create data provider!")
        return None

    return dataProvider


def createRGBRampDataProvider():
    width = 256
    height = 256
    imageDataSize = width * height * 3

    dataP = objc.allocateBuffer(imageDataSize)

    #    Build an image that is RGB 24 bits per sample. This is a ramp
    #    where the red component value increases in red from left to
    #    right and the green component increases from top to bottom.
    #
    idx = 0
    for g in range(height):
        for r in range(width):
            dataP[idx] = chr(r) if sys.version_info[0] == 2 else r
            dataP[idx + 1] = chr(g) if sys.version_info[0] == 2 else g
            dataP[idx + 2] = "\0" if sys.version_info[0] == 2 else 0
            idx += 3

    # Once this data provider is created, the data associated
    # with dataP MUST be available until Quartz calls the data
    # releaser function 'rgbReleaseRampData'.
    dataProvider = Quartz.CGDataProviderCreateWithData(None, dataP, imageDataSize, None)
    return dataProvider


class MyImageDataInfo(object):
    fp = None
    totalBytesRead = 0
    skippedBytes = 0
    numRewinds = 0


def getBytesSequentialAccessDP(data, buffer, count):
    buf = data.fp.read(count)
    buffer[: len(buf)] = buf
    data.totalBytesRead += len(buf)
    return len(buf), buffer


def skipBytesSequentialAccessDP(data, count):
    try:
        data.fp.seek(count, os.SEEK_CUR)
        data.skippedBytes += count

    except IOError as msg:
        print("Couldn't seek %d bytes because of %s" % (count, msg))


def rewindSequentialAccessDP(data):
    # Rewind the beginning of the data.
    data.fp.seek(0, 0)
    data.numRewinds += 1


def releaseSequentialAccessDP(data):
    if data is not None:
        print(
            "read %d bytes, skipped %d bytes, rewind called %d times"
            % (data.totalBytesRead, data.skippedBytes, data.numRewinds)
        )
        data.fp.close()


def createSequentialAccessDPForURL(url):
    success, pathString = Cocoa.CFURLGetFileSystemRepresentation(url, True, None, 1024)
    pathString = pathString.rstrip(b"\0")
    if not success:
        print("Couldn't get the path name C string!")
        return None

    fp = open(pathString, "rb")
    if fp is None:
        print("Couldn't open path to file %s!" % (pathString,))
        return None

    imageDataInfoP = MyImageDataInfo()
    imageDataInfoP.fp = fp

    provider = Quartz.CGDataProviderCreate(
        imageDataInfoP,
        (
            getBytesSequentialAccessDP,
            skipBytesSequentialAccessDP,
            rewindSequentialAccessDP,
            releaseSequentialAccessDP,
        ),
    )
    if provider is None:
        print("Couldn't create data provider!")
        # Release the info data and cleanup.
        releaseSequentialAccessDP(imageDataInfoP)
        return None

    return provider


def getBytesGrayRampDirectAccess(info, buffer, offset, count):
    # This computes a linear gray ramp that is 256 samples wide and
    # 1 sample high. The ith byte in the image is the sample
    # value i. This produces a gray ramp that goes from 0 (black) to
    # FF (white).
    idx = 0

    # This data provider provides 256 bytes total. If Quartz
    # requests more data than is available, only return
    # the available data.
    if (offset + count) > 256:
        count = 256 - offset

    for i in range(offset, offset + count):
        buffer[idx] = chr(i) if sys.version_info[0] == 2 else i
        idx += 1

    return count, buffer


def createGrayRampDirectAccessDP():
    provider = Quartz.CGDataProviderCreateDirectAccess(
        None, 256, (None, None, getBytesGrayRampDirectAccess, None)
    )
    if provider is None:
        print("Couldn't create data provider!")
        return None

    return provider


# This only builds on Tiger and later.
def myCGDataProviderCreateWithCFData(data):
    # If the CFData object passed in is None, this code returns
    # a None data provider.
    if data is None:
        return None

    # Test to see if the Quartz version is available and if so, use it.

    # XXX: force the replacment code to be used
    # if hasattr(Quartz, 'CGDataProviderCreateWithCFData'):
    #    return CGDataProviderCreateWithCFData(data)

    dataSize = Cocoa.CFDataGetLength(data)
    provider = Quartz.CGDataProviderCreateWithData(data, data, dataSize, None)
    return provider


def createDataConsumerFromPathName(path):
    # Create a CFURL for the supplied file system path.
    url = Cocoa.CFURLCreateWithFileSystemPath(
        None, path, Cocoa.kCFURLPOSIXPathStyle, False
    )
    if url is None:
        print("Couldn't create url!")
        return None
    # Create a Quartz data provider for the URL.
    dataConsumer = Quartz.CGDataConsumerCreateWithURL(url)
    if dataConsumer is None:
        print("Couldn't create data consumer!")
        return None

    return dataConsumer


def myCFDataConsumerPutBytes(data, buffer, count):
    # Append 'count' bytes from 'buffer' to the CFData
    # object 'data'.
    Cocoa.CFDataAppendBytes(data, buffer, count)
    return count


# This only builds on Tiger and later.
def myCGDataConsumerCreateWithCFData(data):
    # If the CFData object passed in is None, this code returns
    # a None data consumer.
    if data is None:
        return None

    # Test to see if the Quartz version is available.

    # XXX: force the replacement code to be used:
    # if hasattr(Quartz, 'CGDataConsumerCreateWithCFData'):
    #    return CGDataConsumerCreateWithCFData(data)

    consumer = Quartz.CGDataConsumerCreate(data, (myCFDataConsumerPutBytes, None))
    return consumer
